package net.sf.xanswer.storage.map;

/**
 * The PersistentMap database entry, which maps <code>String</code> key to
 * an array of <code>String</code> values.<br>
 * (<code>String -&gt; String[]</code>)
 */
public interface PersistentEntry {
    
	/**
	 * Gets a key of the current PersistentEntry, which is
	 * associated with the values array of this PersistentEntry.
	 * 
	 * @return a <code>String</code> key of this entry.
	 */	
	String getKey();
	
	/**
	 * Adds some <code>String</code> value within 
	 * the values array of the current PersistentEntry.
	 * 
	 * @param value a <coed>String</code> value 
	 * to associate with a key.
	 * 
	 * @throws PersistentException - if there are problems with
	 * putting a new value to the database.
	 */
	void addValue(String value) throws PersistentException;
	
	/**
	 * Gets the values array within the current PersistentEntry.
	 * 
	 * @return an array of all <code>String</code> entry values.
	 * 
	 * @throws PersistentException - if there are problems with
	 * getting a value from the database.
	 */	
	String[] getValues() throws PersistentException;
	
	/**
	 * Removes the key and all its values from the database.
	 * 
	 * @throws PersistentException - if there are problems with
	 * deleting from the database.
	 */	
	void remove() throws PersistentException;
	
	/**
	 * Gets concrete <code>PersistentMap</code> database, this
	 * <code>PersistentEntry</code> belongs to.
	 * 
	 * @return current <code>PersistentMap</code> database.
	 */	
	PersistentMap getDatabase();
}
