package net.sf.xanswer.storage.map;

import java.net.URI;

/**
 * An <i>Abstract Factory</i> interface for creating and opening
 * certain types of <code>PersistentMap</code> implementation.
 */
public interface PersistentFactory {
	
	/**
	 * Sets root directory where all database files should be placed.
	 * 
	 * @param directory an URI for the root directory
	 * 
	 * @throws IllegalArgumentException if the URI is not a path to
	 * a directory.
	 */
	public void setDirectory(URI directory);
	
	/**
	 * Creates (but NOT opens) the certain implementation of
	 * the <code>PersistentMap</code> interface. For the first time you
	 * should first set the root directory for the factory, using
	 * <code>setDirectory</code> method.
	 * 
	 * @return the PersistentMap implementation to use
	 * 
	 * @throws PersistentException if something went wrong with
	 * creating database or placing files into the specified 
	 * directory
	 * @throws IllegalStateException if the directory was not
	 * previously set using <code>#setDirectory</code> method.
	 */
	public PersistentMap newDatabase() throws PersistentException;

	public PersistentMap newDatabase(String databaseName) throws PersistentException;

    public PersistentMap getDatabase(String databaseName) throws PersistentException;

    /**
     * Gets this factory's name prefix, used in creating database
     * files (or directories).
     * 
     * @return the name prefix for this type of database
     */
    public String getInstanceName();

    // temporary, for Performance tests only?
    public PersistentMap newDatabase(String databaseName, int pageSize, int buffersCount) throws PersistentException;
	
	public static int DEFAULT_PAGE_SIZE = PersistentMap.DEFAULT_PAGE_SIZE;
	public static int DEFAULT_BUFFERS_COUNT = PersistentMap.DEFAULT_BUFFERS_COUNT;
}
